

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.JFrame;


public class Diffusion1D{
	public int screenW=640;
	public int screenH=480;
	Canvas canvas;
	Graphics fg;
	Graphics bg;
	Image image;

	double[] y;
	double[] x;

	double[] yn;
	double Du=3.0; //Diffusion coefficient
	double h=2.0; //delta u (spacial)
	double dt=0.5; //delta t (time)

	//convergence condition
	//c=dt/h*h<=0.5

	int t=0;
	int n;

	public void init() {
		n=640/4;
		y=new double[n];
		x=new double[n];
		yn=new double[n];

		//Initial condition
		for(int i=0;i<n;i++) {
			x[i]=i;
			y[i]=0.0;
		}

		//From center
		for(int i=0;i<10;i++) {
			y[n/2+i]=10.0;
			y[n/2-i]=10.0;
		}

		/*
		//From endpoint
		for(int i=1;i<=10;i++) {
			y[i]=10.0;
			y[i]=10.0;
		}
		*/

	}

	public void draw() {
		//差分法（拡散方程式　右辺）
		for(int i=1;i<n-1;i++) {
			yn[i]=dt*Du*(y[i-1]-2.0*y[i]+y[i+1])/(h*h);
		}
		//値の更新（拡散方程式　左辺）
		for(int i=1;i<n-1;i++) {
			y[i]=y[i]+yn[i];
		}

		/*
		//境界条件（ディレクリ条件）
		y[0]=0.0;
		y[n-1]=0.0;
		*/

		//境界条件（ノイマン条件）
		y[0]=y[1];
		y[n-1]=y[n-2];


		drawScreen();
		t++;
	}

	public void drawScreen() {
		bg.clearRect(0, 0, screenW, screenH);
		bg.setColor(Color.white);
		bg.fillRect(0, 0, screenW, screenH);
		bg.setColor(Color.black);

		int[] pX=new int[n];
		int[] pY=new int[n];

		for(int i=0;i<n;i++) {
			pX[i]=i*4;
			pY[i]=(screenH/2)-(int)(y[i]*20);
		}

		bg.setColor(Color.blue);
		bg.drawPolyline(pX, pY, n);

		//軸の表示
		//X軸
		bg.setColor(Color.lightGray);
		bg.drawLine(
				0,
				screenH/2,
				screenW,
				screenH/2
				);
		bg.drawString("x", screenW-20, screenH/2-20);
		//bg.drawString("X", (int)(axisX[1].x+o.x), (int)(axisX[1].y+o.y));
		//Y軸

		bg.drawLine(
				screenW/2,
				0,
				screenW/2,
				screenH
				);
		bg.drawString("u", screenW/2-20, 20);


		//描画画面にイメージの転送
		fg.drawImage(image, 0, 0, null);
	}

	public Diffusion1D(){
		JFrame f=new JFrame();

		f.setTitle("Diffusion Equation");
		f.setBounds(200,200,screenW,screenH);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		canvas=new Canvas();

		canvas.setSize(screenW,screenH);
		f.add(canvas);
		canvas.setPreferredSize(new Dimension(screenW,screenH));
		f.pack();
		f.setVisible(true);

		image=canvas.createImage(screenW,screenH);
		fg=canvas.getGraphics();
		bg=image.getGraphics();

		init();
		//draw();
		while(true) {
			draw();

		}
	}

	public static void main(String[] args){
		new Diffusion1D();
	}
}